library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

--  Uncomment the following lines to use the declarations that are
--  provided for instantiating Xilinx primitive components.
library UNISIM;
use UNISIM.VComponents.all;

entity opb_xsb300_cf is
  generic (
    C_OPB_AWIDTH    : integer := 32;
    C_OPB_DWIDTH    : integer := 32;
    C_BASEADDR      : std_logic_vector := X"2000_0000";
    C_HIGHADDR      : std_logic_vector := X"2000_00FF"
    );

    Port ( 	OPB_Clk : in std_logic;
                OPB_Rst : in std_logic;
                OPB_ABus : in std_logic_vector (31 downto 0);
                OPB_BE : in std_logic_vector (3 downto 0);
                OPB_DBus : in std_logic_vector (31 downto 0);
                OPB_RNW : in std_logic;
                OPB_select : in std_logic;
                OPB_seqAddr : in std_logic;
                UIO_DBus : out std_logic_vector (31 downto 0);
                UIO_errAck : out std_logic;
                UIO_retry : out std_logic;
                UIO_toutSup : out std_logic;
                UIO_xferAck : out std_logic;
                
                PB_A : out std_logic_vector (19 downto 0);
                PB_WE_N : out std_logic;
                PB_OE_N : out std_logic;
                PB_D : inout std_logic_vector (15 downto 0);
                CS0_N : out std_logic;
                CS1_N : out std_logic;
                DMACK_N : out std_logic;
                --IOCS16_N : in std_logic;-- get rid of this
                --IORDY : in std_logic; -- get rid of this
                INTRQ : in std_logic;
                FPGA_INIT_N : out std_logic

	   );
end opb_xsb300_cf;

architecture Behavioral of opb_xsb300_cf is

component ataCntl
	generic(
		FREQ:				natural := 50_000						-- operating frequency in KHz
	);
	port(
		-- host side
		clk:				in	std_logic;							-- master clock
		rst:				in	std_logic;							-- reset
		rd:					in	std_logic;							-- initiate read operation
		wr:					in	std_logic;							-- initiate write operation
		abort:			in	std_logic;							-- aborts	read/write sector operation
		head:				in	std_logic_vector(3 downto 0);		-- disk head for data access
		cylinder:		in	std_logic_vector(15 downto 0);	-- cylinder for data access
		sector:			in	std_logic_vector(7 downto 0);		-- sector for data access
		hDIn:				in	std_logic_vector(15 downto 0);	-- data from host	to disk
		hDOut:			out	std_logic_vector(15 downto 0);	-- data from disk to host
		done:				out	std_logic;							-- read or write operation is done
		status:			out std_logic_vector(3 downto 0);	-- diagnostic status		

		-- disk side
		dior_n:			out	std_logic;							-- disk register read-enable
		diow_n:			out	std_logic;							-- disk register write-enable
		cs0_n:			out	std_logic;							-- disk command block register select
		cs1_n:			out	std_logic;							-- disk control block register select
		da:					out	std_logic_vector(2 downto 0);		-- register address
		ddIn:				in	std_logic_vector(15 downto 0);	-- data from disk
		ddOut:			out std_logic_vector(15 downto 0);	-- data to disk
		ddOutEnbl:	out	std_logic;							-- enable data outputs to disk
		intrq:			in	std_logic;							-- interrupt from disk
		dmack_n:		out	std_logic								-- DMA acknowledge
	);
end component;

signal cf_rd : std_logic;
signal cf_wr : std_logic;
signal cs, rnw, q0, q1, wq0 : std_logic;

signal addr : std_logic_vector(7 downto 0);
signal wdata : std_logic_vector(15 downto 0);

signal head : std_logic_vector( 3 downto 0 );
signal cylinder: std_logic_vector(15 downto 0);
signal sector: std_logic_vector(7 downto 0);

signal pb_data_tri_n, pb_data_tri : std_logic;
signal pb_data_in : std_logic_vector( 15 downto 0 );
signal pb_data_out : std_logic_vector( 15 downto 0 );
signal diskdata : std_logic_vector( 15 downto 0 );

signal done : std_logic;
signal status : std_logic_vector( 3 downto 0 );
signal state : std_logic;
signal xfer : std_logic;

signal pb_a_i : std_logic_vector(19 downto 0);
signal pb_we_n_i, pb_oe_n_i : std_logic;
signal intrq_i : std_logic;

signal cs0_n_i, cs1_n_i, dmack_n_i : std_logic;
signal start : std_logic;

signal startup :std_logic_vector(2 downto 0) := "000";
signal abort : std_logic;

signal rst : std_logic;


begin

cf : ataCntl port map( 
	-- host side
	clk => OPB_Clk,
	rst => rst,
	rd => '0', --cf_rd,
	wr => '1', --cf_wr,
	abort => '0',
	head => head,
	cylinder => cylinder,
	sector => sector,
	hDIn => X"5657", --OPB_DBus( 15 downto 0 ),
	hDOut => diskdata,
	done => done,
	status => status,

	-- disk side
	dior_n => pb_a_i(18),
	diow_n => pb_a_i(19),
	cs0_n => cs0_n_i,
	cs1_n => cs1_n_i,
	da => pb_a_i( 2 downto 0 ),
	ddIn => pb_data_in,
	ddOut => pb_data_out,
	ddOutEnbl => pb_data_tri_n,
	intrq => intrq_i,
	dmack_n => dmack_n_i
);



FPGA_INIT_N <= '1';
      
UIO_errAck <= '0';
UIO_retry <= '0';
UIO_toutSup <= '0';

process(OPB_Clk, OPB_Rst)
begin
	if OPB_Rst = '1' then
		startup <= "000";
		rst <= '1';
	elsif OPB_Clk'event and OPB_Clk ='1' then
		if startup <= "111" then
			rst <= OPB_Rst;
		else
			rst <='1';
			startup <= startup+1;
		end if;
	end if;
end process;

process(OPB_Clk)
begin
  if OPB_Clk'event and OPB_Clk='1' then
    addr <= OPB_ABus(7 downto 0);
    rnw <= OPB_RNW;
    wdata <= OPB_DBus(31 downto 16);
  end if;
end process;
 

cs <= OPB_select when OPB_ABus(31 downto 20) = X"00A" else '0';

-- state 00 is idle
-- state 01 is accepted transaction
-- state 10 is xfer

process(OPB_Rst, OPB_Clk)
begin
  if OPB_Rst = '1' then
    q0 <= '0';
    q1 <= '0';
  elsif(OPB_Clk'event and OPB_Clk = '1') then
    q0 <= cs and not q1 and not q0;
    q1 <= not q1 and q0;
  end if;  
end process;

UIO_xferAck <= q1;
UIO_DBus <= X"00000000";


head <= X"0";
cylinder <= "0000000000000011";
sector <= X"05";

--state 0 is idle
--state 1 is writing

process(OPB_Rst, OPB_Clk)
begin
  if OPB_Rst = '1' then
    wq0 <= '0';
  elsif OPB_Clk'event and OPB_Clk='1' then
    wq0 <= (q0 and not rnw) or not done;
  end if;
end process;   

cf_wr <= wq0;
cf_rd <= '0';
  
-- I/O
--pb_a_i ( 19 downto 18) <= "10";
pb_a_i ( 17 downto 3 ) <= "00000000000" & done & cf_wr & status(0) & wq0;
pb_oe_n_i <= '0';
pb_we_n_i <= '1';

abuf : for i in 0 to 19 generate
  abuf : OBUF_F_24 port map (
    O => PB_A(i),
    I => pb_a_i(i));  
end generate;
  
webuf :  OBUF_F_24 port map (
    O => PB_WE_N,
    I => pb_we_n_i); 

oebuf :  OBUF_F_24 port map (
    O => PB_OE_N,
    I => pb_oe_n_i); 

cs0buf :  OBUF_F_24 port map (
    O => CS0_N,
    I => cs0_n_i);
cs1buf :  OBUF_F_24 port map (
    O => CS1_N,
    I => cs1_n_i);
mackbuf :  OBUF_F_24 port map (
    O => DMACK_N,
    I => dmack_n_i); 

intrqbuf : IBUF port map (
  I => INTRQ,
  O => intrq_i);

pb_data_tri <= not pb_data_tri_n;       
dbuf : for i in 0 to 15 generate
  dbuf : IOBUF_F_24 port map (
    IO => PB_D(i),
    O => pb_data_in(i),
    I => pb_data_out(i),
    T => pb_data_tri
  );  
end generate;       


end Behavioral;
